class PostData:
    goods_ranking_export = {
        # 'beginDate': '2025-05-20',
        # 'endDate': '2025-05-20',
        'period': '1d',
        'dataType': 1,
        'dataType1': 1,
        'supplier': [],
        'brand': ['639ac2710ac4db00013decc7'],
        'category': '',
        'level': 0,
        'keywords': '',
        'items': [
            'sku_concat_name',
            'brand_name',
            'trd_cnt',
            'per_qty_price',
            'trd_discount_rto',
            'per_usr_trd_qty',
            'per_trd_sku_qty',
            'ord_uv_val',
            'save_click_uv',
            'per_usr_visit_pdtl_pv',
            'inventory_qty',
            'trd_gf_sku_qty',
            'expo_uv',
            'visit_pdtl_uv',
            'per_usr_vist_pdtl_stay_dur',
            'visit_pdtl_rto',
            'add_card_uv',
            'add_cart_uv_rto',
            'trd_usr_num',
            'trd_qty',
            'trd_amt',
            'trd_lfapp_expo_uv_tsfm_rto',
        ],
        'sortItem': 'expo_uv desc',
    }
    brand_channel_export = {
        # 'beginDate': '2025-05-19',
        # 'endDate': '2025-05-19',
        'period': '1d',
        'dataType': 1,
        'supplier': [],
        'brand': ['639ac2710ac4db00013decc7'],
        'items': [
            'usr_atv',
            'per_usr_trd_qty',
            'per_trd_sku_qty',
            'expo_uv',
            'visit_pdtl_uv',
            'visit_pdtl_rto',
            'add_card_uv',
            'add_cart_uv_rto',
            'trd_usr_num',
            'trd_qty',
            'trd_amt',
            'trd_lfapp_expo_uv_tsfm_rto',
            'trd_cnt',
        ],
        'sortItem': 'expo_uv desc',
    }
